@@ -36,6 +36,7 @@ For postgres on Ubuntu this can be done with the following commands:
 
 import collections
 import contextlib
+import fixtures
 
 from alembic import script
 import mock
@@ -56,6 +57,10 @@ from ironic.tests import base
 
 LOG = logging.getLogger(__name__)
 
+# NOTE(vdrok): This was introduced after migration tests started taking more
+# time in gate
+MIGRATIONS_TIMEOUT = 300
+
 
 def _get_connect_string(backend, user, passwd, database):
     """Get database connection
@@ -212,6 +217,8 @@ class MigrationCheckersMixin(object):
         super(MigrationCheckersMixin, self).setUp()
         self.config = migration._alembic_config()
         self.migration_api = migration
+        self.useFixture(fixtures.Timeout(MIGRATIONS_TIMEOUT,
+                                         gentle=True))
 
     def test_walk_versions(self):
         self._walk_versions(self.engine, self.config)
@@ -239,10 +246,9 @@ class MigrationCheckersMixin(object):
 
         self.assertIn('console_enabled', col_names)
         # in some backends bool type is integer
-        self.assertTrue(isinstance(nodes.c.console_enabled.type,
-                                   sqlalchemy.types.Boolean) or
-                        isinstance(nodes.c.console_enabled.type,
-                                   sqlalchemy.types.Integer))
+        self.assertIsInstance(nodes.c.console_enabled.type,
+                              (sqlalchemy.types.Boolean,
+                               sqlalchemy.types.Integer))
 
     def _check_31baaf680d2b(self, engine, data):
         nodes = db_utils.get_table(engine, 'nodes')
@@ -441,10 +447,9 @@ class MigrationCheckersMixin(object):
         self.assertIsInstance(ports.c.portgroup_id.type,
                               sqlalchemy.types.Integer)
         # in some backends bool type is integer
-        self.assertTrue(isinstance(ports.c.pxe_enabled.type,
-                                   sqlalchemy.types.Boolean) or
-                        isinstance(ports.c.pxe_enabled.type,
-                                   sqlalchemy.types.Integer))
+        self.assertIsInstance(ports.c.pxe_enabled.type,
+                              (sqlalchemy.types.Boolean,
+                               sqlalchemy.types.Integer))
 
     def _pre_upgrade_f6fdb920c182(self, engine):
         # add some ports.
@@ -524,6 +529,14 @@ class MigrationCheckersMixin(object):
         self.assertEqual(1, counts['neutron'])
         self.assertEqual(0, counts[None])
 
+    def _check_60cf717201bc(self, engine, data):
+        portgroups = db_utils.get_table(engine, 'portgroups')
+        col_names = [column.name for column in portgroups.c]
+        self.assertIn('standalone_ports_supported', col_names)
+        self.assertIsInstance(portgroups.c.standalone_ports_supported.type,
+                              (sqlalchemy.types.Boolean,
+                               sqlalchemy.types.Integer))
+
     def test_upgrade_and_version(self):
         with patch_with_engine(self.engine):
             self.migration_api.upgrade('head')
@@ -563,6 +576,11 @@ class TestMigrationsPostgreSQL(MigrationCheckersMixin,
 
 class ModelsMigrationSyncMixin(object):
 
+    def setUp(self):
+        super(ModelsMigrationSyncMixin, self).setUp()
+        self.useFixture(fixtures.Timeout(MIGRATIONS_TIMEOUT,
+                                         gentle=True))
+
     def get_metadata(self):
         return models.Base.metadata
 
